@@ -5,7 +5,7 @@
 
 
 # h <- getPortfolioReturns(getIndexComposition('^DJI'), obs=400, reload=TRUE)
-# ws <- optimizePortfolio(h, 350, getCorFilter.RMT() )
+# ws <- optimizePortfolio(h, 350, RandomMatrixDenoiser() )
 # pf <- plotPerformance(h, ws, 350, y.min=-0.4)
 # ef <- compare.EqualWeighted(h, 350, y.min=-0.4)
 # mf <- compare.Market('^GSPC',400,350, y.min=-0.4)
@@ -18,7 +18,7 @@
 #    'ORCL','YHOO','T','AAPL','GOOG','MSFT','IBM','CSCO','INTC',
 #    'GM','F','CAT','MMM','XOM','CVX','RIG','USO','ABX',
 #    'AMGN','HGSI','PFE','JNJ', 'FSLR','STP'), 150, reload=TRUE)
-#  c.gen <- getCorFilter.RMT(hint=c(4,1))
+#  c.gen <- getCorDenoiser.RMT(hint=c(4,1))
 #  weights <- optimizePortfolio(h, 100, c.gen)
 ## End(Not run)
 # NOTE: For zoo compatibility, need to use a t x m matrix for h
@@ -26,54 +26,43 @@
 
 # Optimizes a returns series over a window.
 # s <- c('FCX','AAPL','JPM','AMZN','VMW','TLT','GLD','FXI','ILF','XOM')
-# p <- create(TawnyPortfolio, s)
-# ws <- optimizePortfolio(p, create(RandomMatrixFilter))
-optimizePortfolio %when% (p %isa% TawnyPortfolio)
-optimizePortfolio %as% function(p, estimator)
-{
+# p <- TawnyPortfolio(s)
+# ws <- optimizePortfolio(p, RandomMatrixDenoiser())
+#optimizePortfolio(p, estimator) %::% TawnyPortfolio : a : b
+optimizePortfolio(p, estimator) %when% {
+  p %isa% TawnyPortfolio
+} %as% {
+  flog.debug("[[1]]")
   optimizePortfolio(p, estimator, p.optimize)
 }
 
-optimizePortfolio %when% (p %isa% TawnyPortfolio)
-optimizePortfolio %as% function(p, estimator, optimizer)
-{
+#optimizePortfolio(p, estimator, optimizer) %::% TawnyPortfolio : a : b : c
+optimizePortfolio(p, estimator, optimizer) %when% {
+  p %isa% TawnyPortfolio
+} %as% {
+  flog.debug("[[2]]")
   my.optimizer <- function(p)
   {
-    logger(DEBUG, "Getting correlation matrix")
+    flog.debug("Getting correlation matrix")
     #cor.mat <- cor.gen(h.window, ...)
     cor.mat <- denoise(p, estimator)
 
-    logger(DEBUG, "Optimizing portfolio")
+    flog.debug("Optimizing portfolio")
     optimizer(p$returns, cor.mat)
   }
 
-  logger(INFO, sprintf("Optimizing portfolio for %s-%s",
-         format(start(p)), format(end(p)) ))
+  flog.info("Optimizing portfolio for [%s, %s]",
+         format(start(p)), format(end(p)) )
   ws <- rollapply(p, my.optimizer)
   xts(ws, order.by=as.Date(rownames(ws)))
 }
 
 # This is for backwards compatibility
-#optimizePortfolio <- function(h, window, cor.gen=getCorFilter.RMT(), ...)
-optimizePortfolio %when% TRUE
-optimizePortfolio %as% function(h, window, estimator, ...)
+optimizePortfolio(h, window, estimator, ...) %as%
 {
-  if (! 'zoo' %in% class(h))
-  { cat("WARNING: Zoo objects are preferred for dimensional safety\n") }
-
-  my.optimizer <- function(h.window)
-  {
-    logger(DEBUG, "Getting correlation matrix")
-    #cor.mat <- cor.gen(h.window, ...)
-    cor.mat <- denoise(h, estimator, ...)
-
-    logger(DEBUG, "Optimizing portfolio")
-    p.optimize(h.window, cor.mat)
-  }
-
-  logger(INFO, sprintf("Optimizing portfolio for %s-%s",format(start(h)),format(end(h))))
-  ws <- rollapply(h, window, my.optimizer, by.column=FALSE, align='right')
-  xts(ws, order.by=index(ws))
+  flog.debug("[[3]]")
+  p <- TawnyPortfolio(h, window)
+  optimizePortfolio(p, estimator)
 }
 
 
@@ -93,7 +82,7 @@ p.optimize <- function(h, c.denoised)
   try(c.inv <- solve(c.denoised))
   if (!exists('c.inv'))
   {
-    logger(WARN, "Unable to invert correlation matrix. Returning zeros.")
+    flog.warn("Unable to invert correlation matrix. Returning zeros.")
     return(rep(0, ncol(h)))
   }
 
